import unittest
import pandas as pd
import tensorflow as tf

class TestTitanicClassification(unittest.TestCase):
    def setUp(self):
        dftrain = pd.read_csv('https://storage.googleapis.com/tf-datasets/titanic/train.csv')
        dfeval = pd.read_csv('https://storage.googleapis.com/tf-datasets/titanic/eval.csv')
        y_train = dftrain.pop('survived')
        y_eval = dfeval.pop('survived')

        self.feature_columns = []
        CATEGORICAL_COLUMNS = ['sex', 'n_siblings_spouses', 'parch', 'class', 'deck', 'embark_town', 'alone']
        NUMERIC_COLUMNS = ['age', 'fare']

        for feature_name in CATEGORICAL_COLUMNS:
            vocabulary = dftrain[feature_name].unique()
            self.feature_columns.append(tf.feature_column.categorical_column_with_vocabulary_list(feature_name, vocabulary))

        for feature_name in NUMERIC_COLUMNS:
            self.feature_columns.append(tf.feature_column.numeric_column(feature_name, dtype=tf.float32))

        self.train_input_fn = self.make_input_fn(dftrain, y_train)
        self.eval_input_fn = self.make_input_fn(dfeval, y_eval, num_epochs=1, shuffle=False)

    @staticmethod
    def make_input_fn(data_df=None, label_df=None, num_epochs=10, shuffle=True, batch_size=32):
        def input_function():
            ds = tf.data.Dataset.from_tensor_slices((dict(data_df), label_df))
            if shuffle:
                ds = ds.shuffle(1000)
            ds = ds.batch(batch_size).repeat(num_epochs)
            return ds

        return input_function

    def test_input_fn_output_shape(self):
        train_ds = self.train_input_fn()

    def test_linear_model_evaluation(self):
        linear_est = tf.estimator.LinearClassifier(feature_columns=self.feature_columns)
        linear_est.train(input_fn=self.train_input_fn)

    def test_roc_curve(self):
        linear_est = tf.estimator.LinearClassifier(feature_columns=self.feature_columns)
        linear_est.train(input_fn=self.train_input_fn)

if __name__ == '__main__':
    unittest.main()
